<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Teknisa\FIN\Modules\ContaPagar\Factory\Pag18000CompensacaoTitulosPagarFactory;
use Doctrine\ORM\EntityManager;
use \Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;

class Pag18000CompensacaoTitulosPagarService
{

    const ACRESCIMO = 'A';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    const EMPENHO = 'E';
    protected $crudService;
    protected $entityManager;
    protected $factory;
    protected $fluxoDeCaixaService;
    protected $newCode;
    protected $environment;
    protected $organizacao;

    public function __construct($crudService, EntityManager $entityManager, Pag18000CompensacaoTitulosPagarFactory $factory, FluxoDeCaixaService $fluxoDeCaixaService,
                                NewCode $newCode, Environment $environment)
    {
        $this->crudService = $crudService;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->organizacao = $this->environment->getNrOrgTrab();
    }

    public function geraCompensacao($rowCompensacao, $contasReceber)
    {
        if ($rowCompensacao['IDTPITUTLCOMPAG'] == '0') {
            foreach ($contasReceber as $conta) {
                $this->insereAjusteReceber($this->organizacao, $conta['CDFILIBOLREC'], $conta['NRLANBOLREC'], $conta['VRFINALBREC'],
                    $rowCompensacao['CDAJUCTREC'], $this->environment->getCdOperador(), $rowCompensacao['VRTOTALCOMPPAG']);
                $this->atualizaContasReceber($conta['CDEMPRESA'], $conta['CDFIMOVFLREC'], $conta['CDCLASSFINA'], $conta['CDFILIAL'],
                    $conta['NRLANCTONF'], $conta['NRSEQVENREC'], $conta['NRPARCELAREC'], 'S', $conta['VRFINALBREC'] * -1, $conta['DTMOVFLUXO'],
                    $this->organizacao, $this->environment->getCdOperador(), $rowCompensacao['DTBAIXACOMP']);
            }
        } else {
            foreach ($contasReceber as $conta) {
                $this->atualizaCompensacaoDebito('S', $conta['CDREDUCONT'], $conta['NRMOVIDEBI'], $this->organizacao);
            }
        }
        $this->insereAjustePagar($rowCompensacao['CDFILIBPAG'], $rowCompensacao['NRLANBOLPAG'],
            $rowCompensacao['VRTOTALCOMPPAG'], $rowCompensacao['CDAJUCTPAG'], 'I', $this->organizacao, $this->environment->getCdOperador());
        if ($rowCompensacao['VRTOTALCOMPPAG'] == $rowCompensacao['VRFINALBPAG']) {
            $this->atualizaContasPagar($rowCompensacao['CDEMPRESA'], $rowCompensacao['CDFIMOVFLPAG'], $rowCompensacao['CDCLASSFINA'],
                $rowCompensacao['CDFILIAL'], $rowCompensacao['NRLANCTONF'], $rowCompensacao['NRSEQVENPAG'], $rowCompensacao['NRPARCELAPAG'],
                'S', $rowCompensacao['VRORIGPAG'] * -1, $rowCompensacao['DTMOVFLUXO'], $this->organizacao, $rowCompensacao['CDAJUCTPAG'], $rowCompensacao['DTBAIXACOMP']);

        }else{
            $dtmovfluxo = self::formataData($rowCompensacao['DTMOVFLUXO']);
            $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $this->organizacao, $rowCompensacao['CDEMPRESA'], $rowCompensacao['CDFIMOVFLPAG'], $rowCompensacao['CDCLASSFINA'],
                $dtmovfluxo, $rowCompensacao['VRTOTALCOMPPAG'] * -1, self::EMPENHO, self::SAIDA
                );
            $this->entityManager->persist($movFluxo);
            $this->entityManager->flush();
        }

        if ($rowCompensacao['IDTPITUTLCOMPAG'] == '0') {
            $this->geraBaixaCompensacao($contasReceber, $rowCompensacao['IDTPITUTLCOMPAG'], round($rowCompensacao['VRTOTALCOMPPAG'], 2),
                round($rowCompensacao['VRFINALBPAG'], 2), $rowCompensacao['CDEMPRESA'], $this->environment->getCdFilial(), $rowCompensacao['CDTPBAIXAPAG'],
                $rowCompensacao['DOCUMENTO'], $rowCompensacao['CDREDUCONT'], $rowCompensacao['DTBAIXACOMP'], $rowCompensacao['CDTPBAIXAREC'],
                $rowCompensacao['DSBAIXAREC'], $this->environment->getCdoperador(), $rowCompensacao['CDFILIBPAG'], $rowCompensacao['NRLANBOLPAG'],
                $this->organizacao, $this->environment->getCdOperador(), $rowCompensacao['CDFORNECED'], $rowCompensacao['CDAJUCTREC'], $rowCompensacao['CDAJUCTPAG']);
        }else{
            $this->incluiCompensacao(
                $contasReceber, $rowCompensacao['IDTPITUTLCOMPAG'], $rowCompensacao['CDEMPRESA'], $this->environment->getCdFilial(), $rowCompensacao['CDREDUCONT'],
                $rowCompensacao['CDFORNECED'], null, $rowCompensacao['CDAJUCTPAG'], null, null, $rowCompensacao['CDTPBAIXAPAG'], $rowCompensacao['CDFILIBPAG'],
                $rowCompensacao['NRLANBOLPAG'], $this->environment->getCdOperador(), round($rowCompensacao['VRTOTALCOMPPAG'], 2), null);

        }

    }

    public function insereAjusteReceber($nrorg, $cdfilibrec, $nrlanbolrec, $vrfinalbrec, $cdajuctrec, $operador, $vrtotalcomppag)
    {
        $ajusteBoletoReceber = $this->factory->getAjusteCompBoletoRec($nrorg, $cdfilibrec, $nrlanbolrec, $cdajuctrec);

        $valorCompensado = round(floatval($vrtotalcomppag), 2);

        if ($ajusteBoletoReceber) {
            $vrajbolrec = round(floatval($ajusteBoletoReceber->getVrajbolrec()), 2);
            $valorAjustado = $valorCompensado + $vrajbolrec;
            $ajusteBoletoReceber = $this->factory->updateAjusteBoletoRec($ajusteBoletoReceber, $valorAjustado);
            $this->entityManager->persist($ajusteBoletoReceber);
            $this->entityManager->flush();
        } else {
            $ajusteBoletoReceber = $this->factory->insereAjusteBoletoRec($cdfilibrec, $nrlanbolrec, $cdajuctrec, $vrfinalbrec, '4', $nrorg, $operador);
            $this->entityManager->persist($ajusteBoletoReceber);
            $this->entityManager->flush();
        }
        $boletReceber = $this->factory->getBoletosRec($cdfilibrec, $nrlanbolrec, $this->organizacao);

        if ($boletReceber) {
            $boletReceber = $this->factory->atualizaAjusteBoletoReceber($boletReceber, 0, $nrorg, $operador);
            $this->entityManager->persist($boletReceber);
            $this->entityManager->flush();
        }
    }

    public function deleteAjusteReceber($nrorg, $cdfilibrec, $nrlanbolrec, $vrfinalbrec, $cdajuctrec, $operador, $vrtotalcomppag)
    {
        $ajusteBoletoReceber = $this->factory->getAjusteCompBoletoRec($nrorg, $cdfilibrec, $nrlanbolrec, $cdajuctrec);
        $valorAjustado = $ajusteBoletoReceber->getVrajbolrec();
        $valorCompensado = round(floatval($vrtotalcomppag), 2);
        if ($ajusteBoletoReceber){
            $vrajbolrec = round(floatval($ajusteBoletoReceber->getVrajbolrec()), 2);
            if ($vrajbolrec == $valorCompensado) {
                $this->entityManager->remove($ajusteBoletoReceber);
                $this->entityManager->flush();
            } else {
                $ajbolrec = $this->factory->updateAjusteBoletoRec($ajusteBoletoReceber, $valorCompensado * -1);
                $this->entityManager->persist($ajbolrec);
                $this->entityManager->flush();
            }
        }
        $boletReceber = $this->factory->getBoletosRec($cdfilibrec, $nrlanbolrec, $this->organizacao);
        if ($boletReceber) {
            $this->factory->atualizaAjusteBoletoReceber($boletReceber, $valorAjustado, 'S', $nrorg, $operador);
            $this->entityManager->persist($boletReceber);
            $this->entityManager->flush();
        }
    }

    public function atualizaContasReceber($cdempresa, $cdfimovflrec, $cdclassfina, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec,
                                          $motivo, $vrfinalbrec, $dtmovfluxo, $nrorg, $operador, $dtbaixacomp, $idstatus = 'P')
    {
        $dtmovfluxo = self::formataData($dtmovfluxo);
        $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtmovfluxo, $vrfinalbrec, self::EMPENHO, self::ENTRADA);
        $this->entityManager->persist($movFluxo);
        $this->entityManager->flush();
        $contaReceber = $this->factory->getContaReceber($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg);
        if ($contaReceber) {
            $contaReceber = $this->factory->updateContaReceber($contaReceber, 'E', $idstatus, $dtmovfluxo, $nrorg, $operador);
            $this->entityManager->persist($contaReceber);
            $this->entityManager->flush();
        }
        $ajusteContaReceber = $this->factory->getAjusteContaReceber($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg);
        /*if ($ajusteContaReceber) {
            if ($ajusteContaReceber->getIdajuctrec() == 'A') {
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajusteContaReceber->getCdemmfajcntr(), $ajusteContaReceber->Cdfimfajcntr(),
                    $ajusteContaReceber->getCdclassfina(), $dataBaixa, $ajusteContaReceber->getVrajusterec(),
                    self::EMPENHO, self::ENTRADA);
                $this->entityManager->persist($movFluxo);
                $this->entityManager->flush();
            } else {
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajusteContaReceber->getCdemmfajcntr(), $ajusteContaReceber->Cdfimfajcntr(),
                    $ajusteContaReceber->getCdclassfina(), $ajusteContaReceber->getDtmovfluxo(), $ajusteContaReceber->getVrajusterec() * -1,
                    self::EMPENHO, self::ENTRADA);
                $this->entityManager->persist($movFluxo);
                $this->entityManager->flush();
            }
        }*/
    }

    public function atualizaCompensacaoDebito($idlancomcontpag, $cdreducont, $nrmovidebi, $nrorg)
    {
        $movdebbanc = $this->factory->getMovimentoDebito($cdreducont, $nrmovidebi, $nrorg);
        if ($movdebbanc) {
            $movdebbanc = $this->factory->atualizaCompensacaoDebito($movdebbanc, $idlancomcontpag);
            $this->entityManager->persist($movdebbanc);
            $this->entityManager->flush();
        }
    }

    public function insereAjustePagar($cdfilibpag, $nrlanbolpag, $vrtotalcomppag, $cdajuctpag, $tipoOperacao, $nrorg, $operador)
    {
        $ajusteBoletoPagar = $this->factory->getAjusteBoletoPagar($cdfilibpag, $nrlanbolpag, $cdajuctpag, $this->organizacao);
        $valorCompensado = round(floatval($vrtotalcomppag), 2);
        if ($ajusteBoletoPagar) {
            $vrajbolpag = round(floatval($ajusteBoletoPagar->getVrajbolpag()), 2);
            $valorAjustado = $valorCompensado + $vrajbolpag;
            $ajusteBoletoPagar = $this->factory->updateAjusteBoletoPagar($ajusteBoletoPagar, $valorAjustado);
            $this->entityManager->persist($ajusteBoletoPagar);
            $this->entityManager->flush();
        } else {
            $ajusteBoletoPagar = $this->factory->insereAjusteBoletoPagar($cdfilibpag, $nrlanbolpag, $cdajuctpag, $valorCompensado, 5, $nrorg, $operador);
            $this->entityManager->persist($ajusteBoletoPagar);
            $this->entityManager->flush();
        }
        $boletPagar = $this->factory->getBoletoPagar($cdfilibpag, $nrlanbolpag, $this->organizacao);
        if ($boletPagar) {
            $vrfinalbpag = $boletPagar->getVrfinalbpag() - $valorCompensado;
            $boletPagar = $this->factory->atualizaAjusteBoletoPagar($boletPagar, $vrfinalbpag, 'S', $nrorg, $operador);
            $this->entityManager->persist($boletPagar);
            $this->entityManager->flush();
        }
    }

    public function deleteAjustePagar($cdfilibpag, $nrlanbolpag, $vrtotalcomppag, $cdajuctpag, $tipoOperacao, $nrorg, $operador)
    {
        $ajusteBoletoPagar = $this->factory->getAjusteBoletoPagar($cdfilibpag, $nrlanbolpag, $cdajuctpag, $this->organizacao);
        $valorCompensado = round(floatval($vrtotalcomppag), 2);
        $vrajbolpag = round(floatval($ajusteBoletoPagar->getVrajbolpag()), 2);
        if ($ajusteBoletoPagar){
            if ($vrajbolpag == $valorCompensado) {
                if ($ajusteBoletoPagar) {
                    $this->entityManager->remove($ajusteBoletoPagar);
                    $this->entityManager->flush();
                }
            } else {
                $ajbolpag = $this->factory->updateAjusteBoletoPagar($ajusteBoletoPagar, $valorCompensado * -1);
                $this->entityManager->persist($ajbolpag);
                $this->entityManager->flush();
            }
        }
        $boletPagar = $this->factory->getBoletoPagar($cdfilibpag, $nrlanbolpag, $this->organizacao);
        if ($boletPagar) {
            $vrfinalbpag = $boletPagar->getVrfinalbpag() - ($valorCompensado * -1);
            $boletPagar = $this->factory->atualizaAjusteBoletoPagar($boletPagar, $vrfinalbpag, 'S', $nrorg, $operador);
            $this->entityManager->persist($boletPagar);
            $this->entityManager->flush();
        }

    }

    public function atualizaContasPagar($cdempresa, $cdfimovflpag, $cdclassfina, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag,
                                        $motivo, $vrorigpag, $dtmovfluxo, $nrorg, $cdajuctpag, $dtbaixacomp, $idstatus = 'P')
    {
        $dtmovfluxo = self::formataData($dtmovfluxo);
        if ($motivo != 'S') {
            $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtmovfluxo, $vrorigpag * -1, self::EMPENHO, self::SAIDA);
            $this->entityManager->persist($movFluxo);
            $this->entityManager->flush();
        }

        $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtmovfluxo, $vrorigpag, self::EMPENHO, self::SAIDA);

        $this->entityManager->persist($movFluxo);
        $this->entityManager->flush();

        $contaPagar = $this->factory->getContaPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $nrorg);
        if ($contaPagar) {
            $contaPagar = $this->factory->updateContaPagar($contaPagar, 'E', $idstatus, $dtmovfluxo, $this->organizacao, $this->environment->getCdoperador());
            $this->entityManager->persist($contaPagar);
            $this->entityManager->flush();
        }

        $vrorigpag = round(floatval($vrorigpag), 2);
        $ajusteContaPagar = $this->factory->getTipoAjusteContaPagar($nrorg, $cdfilial, $nrlanctonf, $nrparcelapag, $nrseqvenpag);
        //Atualiza fluxo da conta informada
        if ($ajusteContaPagar) {
            /*if ($ajusteContaPagar->getIdajuctpag() == self::ACRESCIMO) {
                if ($motivo != 'S') {
                    // Desfaz o movimento de fluxo estimado
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                        $nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtbaixacomp, $vrorigpag * -1, self::EMPENHO, self::SAIDA
                        );
                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();
                }
                // Inclui o movimento de fluxo real
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtmovfluxo, $vrorigpag, self::EMPENHO, self::SAIDA);
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
            } else {
                if ($motivo != 'S') {
                    // Desfaz o movimento de fluxo estimado
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtbaixacomp, $vrorigpag, self::EMPENHO, self::SAIDA);
                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();
                }
                // Inclui o movimento de fluxo real
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtmovfluxo, $vrorigpag * -1, self::EMPENHO, self::SAIDA);
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
            }*/
            $ajusteContaPagar = $this->factory->updateAjusteContaPagar($ajusteContaPagar, $dtmovfluxo, $idmovfluxo, $this->organizacao, $this->environment->getCdoperador());
            $this->entityManager->persist($ajusteContaPagar);
            $this->entityManager->flush();
        }
        $contaPagar = $this->factory->getAjusteContaPagar($cdfilial, $nrlanctonf, date_format($dtmovfluxo, 'd/m/Y'), $cdajuctpag, $nrseqvenpag, $nrparcelapag, $this->organizacao);

        if ($contaPagar) {
            $contaPagar = $this->factory->updateAjusteContaPagar($contaPagar, 'E', $dtmovfluxo, $nrorg, $this->environment->getCdOperador());
            $this->entityManager->persist($contaPagar);
            $this->entityManager->flush();
        }


    }

    public function geraBaixaCompensacao($contasReceber, $idtpitutlcompag, $vrtotalcomppag, $vrfinalbpag, $cdempresa,
                                         $filiproc, $cdtpbaixapag, $documento, $cdreducont, $dtbaixacomp, $cdtpbaixarec, $dsbaixarec,
                                         $cdoperlanccomcp, $cdfilibpag, $nrlanbolpag, $nrorg, $operador, $cdforneced, $cdajuctrec, $cdajuctpag)
    {
        $nrbaixapag = null;
        $nrbaixarec = null;
        if ($vrtotalcomppag == $vrfinalbpag) {
            $nrbaixapag = $this->newCode->getCodeByTableName('BAIXAPAGA' . $this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
            $this->geraBaixaPlanejadaPag($cdempresa, $filiproc, $cdtpbaixapag, $documento, $cdreducont, $cdoperlanccomcp, $vrtotalcomppag,
                $cdfilibpag, $nrlanbolpag, $nrorg, $operador, $nrbaixapag);
        }
        if ($idtpitutlcompag == '0') {
            $nrbaixarec = $this->newCode->getCodeByTableName('BAIXARECEBIDA' . $this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
            $nrreciborec = $this->newCode->getCodeByTableName('NRRECIBOREC', $this->environment->getNrOrgTrab());
            $this->geraBaixaPlanejadaRec($contasReceber, $cdreducont, $dtbaixacomp, $cdtpbaixarec, $dsbaixarec,
                $operador, $nrorg, $nrbaixarec, $nrreciborec, $filiproc, $vrtotalcomppag);
        }

        $this->incluiCompensacao($contasReceber, $idtpitutlcompag, $cdempresa, $filiproc, $cdreducont, $cdforneced, $cdtpbaixarec,
            $cdajuctpag, $nrbaixapag, $nrbaixarec, $cdtpbaixapag, $cdfilibpag, $nrlanbolpag, $cdoperlanccomcp, $vrtotalcomppag, $cdajuctrec);
    }

    public function geraBaixaPlanejadaPag($cdempresa, $filiproc, $cdtpbaixapag, $documento, $cdreducont, $cdoperlanccomcp, $valorbaixa,
                                          $cdfilibpag, $nrlanbolpag, $nrorg, $operador, $nrbaixapag)
    {
        $baixaPaga = $this->factory->insereBaixaPaga(
            $cdempresa, $filiproc, $cdtpbaixapag, $documento, $cdreducont,
            $valorbaixa, $cdfilibpag, $nrlanbolpag, $nrorg, $operador, $nrbaixapag
        );
        $this->entityManager->persist($baixaPaga);
        $boletoPagar = $this->factory->getBoletoPagar($cdfilibpag, $nrlanbolpag, $nrorg);
        if ($boletoPagar){
            $boletPagar = $this->factory->atualizaBoletoPagar($boletoPagar, 'P', '00', $nrbaixapag, $filiproc, $cdtpbaixapag, $operador, $nrorg);
            $this->entityManager->persist($boletoPagar);
            $this->entityManager->flush();
        }
    }

    public function geraBaixaPlanejadaRec($contasReceber, $cdreducont, $dtbaixacomp, $cdtpbaixarec, $dsbaixarec,
                                          $operador, $nrorg, $nrbaixarec, $nrreciborec, $filiproc, $vrtotalcomppag)
    {
        $dtbaixacomp = self::formataData($dtbaixacomp);
        $baixaRecebida = $this->factory->insereBaixaRec($contasReceber[1]['CDEMPRESA'], null, null, $nrbaixarec, $nrreciborec, 'P', $vrtotalcomppag, $operador, $cdreducont,
            $dtbaixacomp, $filiproc, $cdtpbaixarec, $dsbaixarec, $nrorg);
        $this->entityManager->persist($baixaRecebida);
        $this->entityManager->flush();
        foreach ($contasReceber as $boleto) {
            $boletosReceber = $this->factory->getBoletosRec($boleto['CDFILIBOLREC'], $boleto['NRLANBOLREC'], $nrorg);
            if ($boletosReceber) {
                $boletoReceber = $this->factory->atualizaBoletosRec($boletosReceber, 'P', $nrbaixarec, $filiproc, $cdtpbaixarec, $nrorg, $operador);
                $this->entityManager->persist($boletoReceber);
            }
        }
    }


    public function incluiCompensacao($contas, $idtpitutlcompag, $cdempresa, $filiproc, $cdreducont, $cdforneced, $cdtpbaixarec,
                                      $cdajuctpag, $nrbaixapag, $nrbaixarec, $cdtpbaixapag, $cdfilibpag, $nrlanbolpag, $cdoperlanccomcp, $vrtotalcomppag, $cdajuctrec)
    {

        $nrcompensacao = $this->newCode->getCodeByTableName('COMPCONTPAG'.$cdempresa, $this->environment->getNrOrgTrab());
        $compensacao = $this->factory->insereCompensacontpag($cdempresa, $nrcompensacao, $filiproc, $nrlanbolpag, $cdoperlanccomcp, $cdforneced,
            $cdtpbaixarec, $cdtpbaixapag, $cdajuctrec, $cdajuctpag, $cdreducont, $filiproc, $nrbaixarec, $cdfilibpag, $nrbaixapag,
            $idtpitutlcompag, count($contas), $vrtotalcomppag, $this->organizacao, $this->environment->getCdOperador());
        $this->entityManager->persist($compensacao);
        $this->entityManager->flush();
        $sequencialItem = 0;
        if ($idtpitutlcompag == '0') {
            foreach ($contas as $conta) {
                $sequencialItem = intval($sequencialItem) + 1;
                $sequencialItem = str_pad($sequencialItem, 5, '0', STR_PAD_LEFT);
                $itemCompensado = $this->factory->insereItensCompensacaoPagar($cdempresa, $nrcompensacao, $sequencialItem, $conta['CDFILIBOLREC'],
                    $conta['NRLANBOLREC'], null, null, $conta['VRFINALBREC'], $this->organizacao, $this->environment->getCdOperador());
                $this->entityManager->persist($itemCompensado);
                $this->entityManager->flush();
            }
        } else {
            foreach ($contas as $lancamento) {
                $sequencialItem = intval($sequencialItem) + 1;
                $sequencialItem = str_pad($sequencialItem, 5, '0', STR_PAD_LEFT);
                $itemCompensado = $this->factory->insereItensCompensacaoPagar($cdempresa, $nrcompensacao, $sequencialItem, null,
                    null, $lancamento['CDREDUCONT'], $lancamento['NRMOVIDEBI'], $lancamento['VRMOVIDEBI'], $this->organizacao, $this->environment->getCdOperador());
                $this->entityManager->persist($itemCompensado);
                $this->entityManager->flush();
            }
        }
    }

    public function deleteCompensacaoPagar($row, $contasReceber)
    {
        $boletoPagar = $this->factory->getBoletoPagar($row['CDFILIBPAG'], $row['NRLANBOLPAG'], $this->organizacao);
        if ($this->validaPermissao($row, $boletoPagar)) {
            $baixaPaga = $this->factory->getBaixapaga($row['CDFILIBPAG'], $row['NRBAIXAPAG'], $this->organizacao);
            $baixaRecebida = $this->factory->getBaixaRecebida($row['CDFILIBAIXAREC'], $row['NRBAIXAREC'], $this->organizacao);
            /*if ($baixaPaga->getIdsitucnabp != ''){
                Throw new |Exception('Exclusão não permitida! Baixa a pagar não está planejada.');
            }
            if ($baixaRecebida->getIdsitucnabp != ''){
                Throw new |Exception('Exclusão não permitida! Baixa a pagar não está planejada.');
            }*/
            $this->deleteAjustePagar($row['CDFILIBPAG'], $row['NRLANBOLPAG'],
                $row['VRTOTALCOMPPAG'], $row['CDAJUCTPAG'], 'I', $this->organizacao, $this->environment->getCdOperador());
            if ($row['VRTOTALCOMPPAG'] == $row['VRFINALBPAG']) {
                $this->atualizaContasPagar($row['CDEMPRESA'], $row['CDFIMOVFLPAG'], $row['CDCLASSFINA'],
                    $row['CDFILIAL'], $row['NRLANCTONF'], $row['NRSEQVENPAG'], $row['NRPARCELAPAG'],
                    'S', $row['VRORIGPAG'], $row['DTMOVFLUXO'], $row['NRORG'], $row['CDAJUCTPAG'], 'A');
            }else{
                $dtmovfluxo = self::formataData($row['DTMOVFLUXO']);
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $this->organizacao, $row['CDEMPRESA'], $row['CDFIMOVFLPAG'], $row['CDCLASSFINA'],
                    $dtmovfluxo, $row['VRTOTALCOMPPAG'], self::EMPENHO, self::SAIDA
                    );
                $this->entityManager->persist($movFluxo);
                    $this->entityManager->flush();
            }
            $boletoPagar = $this->factory->atualizaBoletoPagar($boletoPagar, 'A', '00', null, null, null, $this->environment->getCdOperador(), $this->organizacao);
            $this->entityManager->persist($boletoPagar);
            $this->entityManager->flush();

            if ($row['IDTPITUTLCOMPAG'] == '0') {
                foreach ($contasReceber as $conta) {
                    $this->deleteAjusteReceber($this->organizacao, $conta['CDFILIBOLREC'], $conta['NRLANBOLREC'], $conta['VRORIGREC'],
                    $row['CDAJUCTREC'], $this->environment->getCdOperador(), $row['VRTOTALCOMPPAG']);

                    $this->atualizaContasReceber($conta['CDEMPRESA'], $conta['CDFIMOVFLREC'], $conta['CDCLASSFINA'], $conta['CDFILIAL'],
                    $conta['NRLANCTONF'], $conta['NRSEQVENREC'], $conta['NRPARCELAREC'], 'S', $conta['VRITEMCOMPPAG'], $conta['DTMOVFLUXO'],
                    $this->organizacao, $this->environment->getCdOperador(), $row['DTBAIXACOMP']);

                    $boletoReceber = $this->factory->getBoletosRec($conta['CDFILIBOLREC'], $conta['NRLANBOLREC'], $this->organizacao);

                    if ($boletoReceber) {
                        $boletoReceber = $this->factory->atualizaBoletosRec($boletoReceber, 'A', null, null, null, $this->organizacao, $this->environment->getCdOperador());
                        $this->entityManager->persist($boletoReceber);
                        $this->entityManager->flush();
                    }
                }
            } else {
                foreach ($contasReceber as $contarec) {
                    $this->atualizaCompensacaoDebito('N', $contarec['CDREDUCONT'], $contarec['NRMOVIDEBI'], $this->organizacao);
                }
            }
            $itensCompensados = $this->factory->getItensCompensados($row['CDEMPRESA'], $row['NRLANCCOMPPAG'], $this->organizacao);
            foreach ($itensCompensados as $item) {
                $this->entityManager->remove($item);
                $this->entityManager->flush();
            }
            $compensacao = $this->factory->getCompensacao($row['CDEMPRESA'], $row['NRLANCCOMPPAG'], $this->organizacao);
            $this->entityManager->remove($compensacao);
            if ($row['IDTPITUTLCOMPAG'] == '0') {
                if ($baixaPaga) {
                    if ($baixaPaga->getCdempresa() && $baixaPaga->getNrlctbbpag()) {
                        $lanccont = $this->factory->getLancamentoContabil($row['CDEMPRESA'], $row['NRLCTBBPAG'], $this->organizacao);
                        if ($lanccont) {
                            $this->entityManager->remove($lanccont);
                            $this->entityManager->flush();
                        }
                    }
                }
                if ($baixaPaga) {
                    $this->entityManager->remove($baixaPaga);
                    $this->entityManager->flush();
                }

                if ($baixaRecebida->getCdempresa() && $baixaRecebida->getNrlctbbrec()) {
                    $lanccont = $this->factory->getLancamentoContabil($row['CDEMPRESA'], $row['NRLCTBBREC'], $this->organizacao);
                    if ($lanccont) {
                        $this->entityManager->remove($lanccont);
                        $this->entityManager->flush();
                    }
                }
                if ($baixaRecebida) {
                    $this->entityManager->remove($baixaRecebida);
                    $this->entityManager->flush();
                }
            }
        }
    }


    public function validaPermissao($row, $boletPagar)
    {
        if ((($boletPagar->getIdstatbpag() == 'B') || ($boletPagar->getIdstatbpag() == 'P'))
            && $row['NRBAIXAPAG'] == ''
        ) {
            throw new \Exception('Exclusão não permitida! Boleto a pagar ligado à compensação já foi baixado.');
        }
        if (($boletPagar->getIdsitucnabp() <> '00') && ($boletPagar->getIdsitucnabp() <> '01')) {
            throw new \Exception('Exclusão não permitida! Boleto a pagar ligado à compensação foi enviado via CNAB.');
        }
        //Validação de Antecipação de fornecedores
        if ($boletPagar->getNrlancantecipa()) {
            throw new \Exception('Exclusão não permitida! Boleto a pagar ligado a uma antecipação.');
        }
        return true;
    }

    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}
?>