<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Financeiro\Entities\Movreal;
use Teknisa\Model\Financeiro\Entities\Concimovbanc;

class Bco54500ConciliacaoFactory{

    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function updateMovreal($statusConciliacao, $formaConciliacao, $nrconcimovb, $cdsequextr,
                                  $nrseqmovreal, $cdreducont, $nrorg, $nrmovbare)
    {
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdreducont'] = $cdreducont;
            $params['cdsequextr'] = $cdsequextr;
            $params['nrseqmovreal'] = $nrseqmovreal;
            $movreal = $this->entityManager->getRepository(Movreal::class)->findOneBy($params);
            $movreal->setIdstconmvban($statusConciliacao);
            $movreal->setIdconcreal($formaConciliacao);
            $movreal->setNrmovbare($nrmovbare);
            $movreal->setNrconcimovb($nrconcimovb);
            $movreal->setDtultatu(DateUtil::getDataAtual());
            $movreal->setNrorg($nrorg);
            $movreal->setNrorgultatu($nrorg);
            return $movreal;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function updateMovbancario($statusConciliacao, $formaConciliacao, $nrconcimovb, $nrmovbanc, $cdreducont, $nrorg, $cdsequexba, $nrseqmovreba)
    {
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdreducont'] = $cdreducont;
            $params['nrmovbanc'] = $nrmovbanc;
            $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);
            $movbancario->setIdstconext($statusConciliacao);
            $movbancario->setIdconcbanc($formaConciliacao);
            $movbancario->setCdsequexba($cdsequexba);
            $movbancario->setNrseqmovreba($nrseqmovreba);
            $movbancario->setNrconcimovb($nrconcimovb);
            $movbancario->setDtultatu(DateUtil::getDataAtual());
            $movbancario->setNrorg($nrorg);
            $movbancario->setNrorgultatu($nrorg);
            return $movbancario;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function insereConcimovbanc($cdreducont, $nrorg, $nrconcimovb){
        $concimovbanc = new Concimovbanc();
        $concimovbanc->setNrconcimovb($nrconcimovb);
        $concimovbanc->setCdreducont($cdreducont);
        $concimovbanc->setNrorg($nrorg);
        $concimovbanc->setDtinclusao(DateUtil::getDataAtual());
        $concimovbanc->setNrorginclusao($nrorg);
        return $concimovbanc;
    }

    public function getConcimovbanc($cdreducont, $nrorg, $nrconcimovb){
        try {
            $params = array();
            $params['nrorg'] = $nrorg;
            $params['cdreducont'] = $cdreducont;
            $params['nrconcimovb'] = $nrconcimovb;
            $concimovbanc = $this->entityManager->getRepository(Concimovbanc::class)->findOneBy($params);
            return $concimovbanc;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getMovsForConcili($nrdia, $tdtconcilia, $dtconcilia, $cdreducont) { 
        $sql = "SELECT ABS(MOVREAL.DTMOVREAL - MOVBANCARIO.DTMOVBANC) AS TOTALDIA,
                  MOVREAL.NRSEQMOVREAL,
                  MOVREAL.NRDOCMOVREAL,
                  MOVREAL.DTMOVREAL,
                  MOVREAL.VRMOVREAL,
                  MOVREAL.IDSTCONMVBAN,
                  MOVREAL.CDSEQUEXTR,
                  MOVREAL.CDREDUCONT,
                  MOVBANCARIO.NRMOVBANC,
                  MOVBANCARIO.DTMOVBANC,
                  MOVBANCARIO.IDMOVBANC,
                  MOVBANCARIO.NRDOCMBANC,
                  MOVBANCARIO.DSHISMBANC,
                  MOVBANCARIO.VRMOVBANC,
                  MOVBANCARIO.IDSTCONEXT
                FROM MOVREAL,
                  MOVBANCARIO
                WHERE (MOVREAL.CDREDUCONT               = :CDREDUCONT)
                AND (MOVREAL.IDSTCONMVBAN               = '0')
                AND (MOVBANCARIO.CDREDUCONT             = :CDREDUCONT)
                AND (MOVBANCARIO.IDSTCONEXT             = '0')
                AND (MOVBANCARIO.IDMOVBANC              = MOVREAL.IDMOVREAL)
                AND (ROUND(MOVBANCARIO.VRMOVBANC, 2)    = ROUND(MOVREAL.VRMOVREAL, 2))
                AND ((MOVBANCARIO.NRDOCMBANC           IS NULL)
                OR (LPAD(MOVBANCARIO.NRDOCMBANC, 20, 0) = LPAD(MOVREAL.NRDOCMOVREAL, 20, 0)))
                AND ((MOVBANCARIO.DTMOVBANC             = :DTCONCILIA)
                OR (:TDTCONCILIA                        = 'T'))
                AND ((MOVREAL.DTMOVREAL BETWEEN MOVBANCARIO.DTMOVBANC - :NRDIA AND MOVBANCARIO.DTMOVBANC + :NRDIA
                AND :TDTCONCILIA      = 'T')
                OR (MOVREAL.DTMOVREAL = :DTCONCILIA))
                ORDER BY 1";

        $params = array("NRDIA"=>$nrdia,
                        "TDTCONCILIA"=>$tdtconcilia,
                        "DTCONCILIA"=>$dtconcilia,
                        "CDREDUCONT"=>$cdreducont);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function updateMovrealAuto($nrmovbanc, $cdreducont, $nrseqmovreal, $cdsequextr) {
        $sql="UPDATE MOVREAL
                SET IDSTCONMVBAN  = '1',
                  IDCONCREAL      = 'A',
                  NRMOVBARE       = :NRMOVBARE
                WHERE (CDREDUCONT = :CDREDUCONT)
                AND (NRSEQMOVREAL = :NRSEQMOVREAL)
                AND (CDSEQUEXTR   = :CDSEQUEXTR)";
                
        $params = array("CDREDUCONT"=>$cdreducont,
                        "CDSEQUEXTR"=>$cdsequextr,
                        "NRSEQMOVREAL"=>$nrseqmovreal,
                        "NRMOVBARE"=>$nrmovbanc);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    public function updateMovBancaio($cdsequextr, $nrseqmovreal, $cdreducont, $nrmovbanc) {
        $sql="UPDATE MOVBANCARIO
                SET IDSTCONEXT    = '1',
                  IDCONCBANC      = 'A',
                  CDSEQUEXBA      = :CDSEQUEXTR,
                  NRSEQMOVREBA    = :NRSEQMOVREAL
                WHERE (CDREDUCONT = :CDREDUCONT)
                AND (NRMOVBANC    = :NRMOVBANC)";
                
        $params = array("CDREDUCONT"=>$cdreducont,
                        "CDSEQUEXTR"=>$cdsequextr,
                        "NRSEQMOVREAL"=>$nrseqmovreal,
                        "NRMOVBANC"=>$nrmovbanc);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    public function getMoreLancs($nrdia, $tdtconcilia, $dtconcilia, $cdreducont) {
        $sql = "SELECT MOVREAL.DTMOVREAL
                FROM MOVREAL,
                  MOVBANCARIO
                WHERE (MOVREAL.CDREDUCONT   = :CDREDUCONT)
                AND (MOVREAL.IDSTCONMVBAN   = '0')
                AND (MOVBANCARIO.CDREDUCONT = :CDREDUCONT)
                AND (MOVBANCARIO.IDSTCONEXT = '0')
                AND (MOVBANCARIO.IDMOVBANC  = MOVREAL.IDMOVREAL)
                AND ((MOVREAL.DTMOVREAL BETWEEN MOVBANCARIO.DTMOVBANC - :NRDIA AND MOVBANCARIO.DTMOVBANC + :NRDIA
                AND :TDTCONCILIA    = 'T')
                OR (MOVREAL.DTMOVREAL = :DTCONCILIA))
                AND (ROWNUM           = 1)
                ORDER BY 1";

        $params = array("NRDIA"=>$nrdia,
                        "TDTCONCILIA"=>$tdtconcilia,
                        "DTCONCILIA"=>$dtconcilia,
                        "CDREDUCONT"=>$cdreducont);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function verifyRegisForConcili($nrdia, $tdtconcilia, $dtconcilia, $cdreducont) {
        $sql = "SELECT MOVREAL.DTMOVREAL
                FROM MOVREAL,
                  MOVBANCARIO
                WHERE (MOVREAL.CDREDUCONT   = :CDREDUCONT)
                AND (MOVREAL.IDSTCONMVBAN   = '0')
                AND (MOVBANCARIO.CDREDUCONT = :CDREDUCONT)
                AND (MOVBANCARIO.IDSTCONEXT = '0')
                AND (MOVBANCARIO.IDMOVBANC  = MOVREAL.IDMOVREAL)
                AND ((MOVREAL.DTMOVREAL BETWEEN MOVBANCARIO.DTMOVBANC - :NRDIA AND MOVBANCARIO.DTMOVBANC + :NRDIA
                AND :TDTCONCILIA    = 'T')
                OR (MOVREAL.DTMOVREAL = :DTCONCILIA))
                AND (ROWNUM           = 1)
                ORDER BY 1";

        $params = array("NRDIA"=>$nrdia,
                        "TDTCONCILIA"=>$tdtconcilia,
                        "DTCONCILIA"=>$dtconcilia,
                        "CDREDUCONT"=>$cdreducont);
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    /**
     * Get Saldo Conta Corrente
     * @param $cdreducont
     * @param $datarefe
     * @return count
     */
    public function getSaldoContaCorr($cdreducont, $datarefe){
        $params = array();
        
        $params['CDREDUCONT'] = $cdreducont;
        $params['DATAREFE']   = $datarefe;
        
        $sql = "SELECT SUM(VRSDOCCORR) SALDO
                  FROM SDOCCORR
                 WHERE CDREDUCONT = :CDREDUCONT AND
                       DTSDOCCORR = (SELECT MAX(DTSDOCCORR)
                                       FROM SDOCCORR
                                      WHERE CDREDUCONT = :CDREDUCONT AND
                                            DTSDOCCORR < :DATAREFE)";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
}
